#include "cflironecontrol.h"
//GUID 
#include <initguid.h>
DEFINE_GUID(GUID_DEVINTERFACE_FLIRONEGEN2_IAP,0x41a83025, 0x9dc8, 0x4e75, 0x87, 0x5, 0x12, 0x9, 0x22, 0x2f, 0xdc, 0x17);
DEFINE_GUID(GUID_DEVINTERFACE_FLIRONEGEN2_FRAME,0x1b2e6756, 0x344, 0x449e, 0x8b, 0x5b, 0xbd, 0x57, 0xf5, 0x82, 0x76, 0x6b);
DEFINE_GUID(GUID_DEVINTERFACE_FLIRONEGEN2_FILEIO,0xaa0dfc0c, 0xe043, 0x4778, 0x9d, 0xba, 0x28, 0x74, 0x9a, 0x3a, 0xa9, 0xf5);

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//- ---------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CFlirOneControl::CFlirOneControl(void)
{ 
 long n;

 SizeOfData=IMAGE_BUFFER_SIZE;
 Data=new char[SizeOfData]; 

 ImageBuffer=new unsigned char[IMAGE_BUFFER_SIZE];
 ImageBufferSize=0;

 FrameIndex=0;

 ThermalImage=new unsigned short[THERMAL_IMAGE_SIZE_SHORT];
 ColorImage=new unsigned long[COLOR_IMAGE_SIZE_LONG];

 for(n=0;n<IMAGE_WIDTH*IMAGE_HEIGHT;n++)
 {
  ThermalImage[n]=0x00;
  ColorImage[n]=0xFF808080;
 }

 //  -
 for(n=0;n<256;n++)
 {
  ColorMap_R[n]=(unsigned char)n;
  ColorMap_G[n]=(unsigned char)n;
  ColorMap_B[n]=(unsigned char)n;
 } 

 hFile_FileIO=INVALID_HANDLE_VALUE;
 hFile_Frame=INVALID_HANDLE_VALUE;
 hFile_iAP=INVALID_HANDLE_VALUE;

 Overlapped_Frame.Internal=0;
 Overlapped_Frame.InternalHigh=0;
 Overlapped_Frame.Offset=0;
 Overlapped_Frame.OffsetHigh=0;
 Overlapped_Frame.hEvent=CreateEvent(NULL,TRUE,FALSE,NULL);

 Overlapped_FileIO.Internal=0;
 Overlapped_FileIO.InternalHigh=0;
 Overlapped_FileIO.Offset=0;
 Overlapped_FileIO.OffsetHigh=0;
 Overlapped_FileIO.hEvent=CreateEvent(NULL,TRUE,FALSE,NULL);

 Overlapped_iAP.Internal=0;
 Overlapped_iAP.InternalHigh=0;
 Overlapped_iAP.Offset=0;
 Overlapped_iAP.OffsetHigh=0;
 Overlapped_iAP.hEvent=CreateEvent(NULL,TRUE,FALSE,NULL);

 Start_FileIO=true;
 Start_Frame=true;
 Start_iAP=true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//- ----------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CFlirOneControl::~CFlirOneControl()
{
 DeleteObject(Overlapped_Frame.hEvent);
 DeleteObject(Overlapped_FileIO.hEvent);
 DeleteObject(Overlapped_iAP.hEvent);
 delete[](Data);
 delete[](ImageBuffer);
 delete[](ThermalImage);
 delete[](ColorImage);
 Close();
}
//- ------------------------------------------------------------

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------------------------------------------------------------
// crc
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::CalculateCRC(unsigned short &crc,unsigned char byte)
{
 crc^=(byte<<8);
 for(unsigned char n=0;n<8;n++) 
 { 
  if (crc&0x8000) crc=(crc<<1)^0x1021;
             else crc<<=1;
 }
}

//----------------------------------------------------------------------------------------------------
//   log-
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::AddLog(char *string)
{
 return;
 FILE *file=fopen("log.txt","ab");
 if (file==NULL) return;
 fprintf(file,"%s",string);
 fclose(file);
}
//----------------------------------------------------------------------------------------------------
//   log-
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::AddLog(long value)
{
 return;
 FILE *file=fopen("log.txt","ab");
 if (file==NULL) return;
 fprintf(file,"%i",value);
 fclose(file);
}
//----------------------------------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::SendControlCommand(unsigned long code)
{
 unsigned long nBytes;
 char buffer[256];
 unsigned long size=sizeof(buffer);  
 if (hFile_iAP==INVALID_HANDLE_VALUE) return(false);
 if (DeviceIoControl(hFile_iAP,code,buffer,size,buffer,size,(PULONG)&nBytes,NULL)==FALSE)
 {
  if (GetLastError()!=ERROR_IO_PENDING) 
  {
   AddLog("[]   DeviceIoControl   ");
   AddLog(code);
   AddLog("\r\n");
   Close();
   return(false);  
  }
 }
 else return(true);
 if (WaitForSingleObject(Overlapped_iAP.hEvent,500)==WAIT_OBJECT_0)// 
 {
  AddLog("Event iAP!\r\n");
  if (GetOverlappedResult(hFile_iAP,&Overlapped_iAP,&nBytes,TRUE)==TRUE) return(true);
 }
 return(true);// 
}


//----------------------------------------------------------------------------------------------------
//   GUID
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::GetDevicePath(IN LPGUID InterfaceGuid,PCHAR DevicePath,size_t BufLen)
{
 HDEVINFO HardwareDeviceInfo;
 SP_DEVICE_INTERFACE_DATA DeviceInterfaceData;
 PSP_DEVICE_INTERFACE_DETAIL_DATA DeviceInterfaceDetailData=NULL;
 ULONG Length,RequiredLength=0;
 BOOL bResult;

 HardwareDeviceInfo=SetupDiGetClassDevs(InterfaceGuid,NULL,NULL,(DIGCF_PRESENT|DIGCF_DEVICEINTERFACE));
 if (HardwareDeviceInfo==INVALID_HANDLE_VALUE) 
 {
  AddLog("[]   SetupDiGetClassDevs!\r\n");
  return(false);
 }

 DeviceInterfaceData.cbSize=sizeof(SP_DEVICE_INTERFACE_DATA);
 bResult=SetupDiEnumDeviceInterfaces(HardwareDeviceInfo,0,InterfaceGuid,0,&DeviceInterfaceData);
 if (bResult==FALSE) 
 {
  AddLog("[]   SetupDiEnumDeviceInterfaces!\r\n");
  SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
  return(false);
 }
 SetupDiGetDeviceInterfaceDetail(HardwareDeviceInfo,&DeviceInterfaceData,NULL,0,&RequiredLength,NULL);
 DeviceInterfaceDetailData=(PSP_DEVICE_INTERFACE_DETAIL_DATA)
 LocalAlloc(LMEM_FIXED,RequiredLength);

 if (DeviceInterfaceDetailData==NULL) 
 {
  SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
  AddLog("[]   !\r\n");
  return(false);
 }
 DeviceInterfaceDetailData->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
 Length=RequiredLength;

 bResult=SetupDiGetDeviceInterfaceDetail(HardwareDeviceInfo,&DeviceInterfaceData,DeviceInterfaceDetailData,Length,&RequiredLength,NULL);
 if (bResult==FALSE) 
 {
  SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
  AddLog("[]   SetupDiGetDeviceInterfaceDetail!\r\n");
  return(false);
 }
 if (strlen(DeviceInterfaceDetailData->DevicePath)>=BufLen-1)
 {
  SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
  LocalFree(DeviceInterfaceDetailData);
  return(false);
 }
 strcpy(DevicePath,DeviceInterfaceDetailData->DevicePath);
 SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
 LocalFree(DeviceInterfaceDetailData);
 return(true);
}



//----------------------------------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::CreateImage(unsigned char *buffer,unsigned long size)
{
 unsigned long n;
 long x;
 long y;
 if (size+ImageBufferSize>=IMAGE_BUFFER_SIZE) ImageBufferSize=0;//  
 // 
 for(n=0;n<size;n++) ImageBuffer[n+ImageBufferSize]=buffer[n];
 ImageBufferSize+=size; 
 if (ImageBufferSize<4) return(false);

 //     4  ,     
 unsigned char magicbyte[4]={0xEF,0xBE,0x00,0x00};
 for(n=0;n<ImageBufferSize-4;n++)
 {
  long m;
  bool ok=true;
  for(m=0;m<4;m++)
  {
   if (ImageBuffer[n+m]==magicbyte[m]) continue;
   ok=false;
   break;
  }
  if (ok==true)// 
  {
   //   
   long size=ImageBufferSize-n-4;
   for(m=0;m<size;m++) ImageBuffer[m]=ImageBuffer[m+n+4];
   ImageBufferSize=size;
   n=0;//  
  }
 }
 if (ImageBufferSize<24) return(false);//  
 unsigned long FrameSize=*((unsigned long *)(ImageBuffer+8));
 unsigned long ThermalSize=*((unsigned long *)(ImageBuffer+12));
 unsigned long JpgSize=*((unsigned long *)(ImageBuffer+16));
 unsigned long StatusSize=*((unsigned long *)(ImageBuffer+20));
 if (FrameSize+28>ImageBufferSize) return(false);//    

 // 
 ImageBufferSize=0;
 for(n=0;n<IMAGE_WIDTH*IMAGE_HEIGHT;n++)
 {
  ThermalImage[n]=0x00;
  ColorImage[n]=0xFF808080;
 }
 //   (164   )
 long min=0x10000;
 long max=0x0;
 unsigned short *d_ptr=(unsigned short*)(ImageBuffer+24);
 bool enabled=true;
 for(y=0;y<ORIGINAL_IMAGE_HEIGHT;y++) 
 {  
  unsigned short crc=0;
  unsigned short id;
  unsigned short crc_package;
  // 
  id=*d_ptr;
  id&=0x0FFF;
  CalculateCRC(crc,(id>>8)&0xFF);
  CalculateCRC(crc,id&0xFF);
  d_ptr++;
  crc_package=*d_ptr;  
  CalculateCRC(crc,0);
  CalculateCRC(crc,0);
  d_ptr++;
  for(x=0;x<ORIGINAL_IMAGE_WIDTH/2;x++,d_ptr++)
  {   
   unsigned short value=*d_ptr;
   unsigned long offset=y+(ORIGINAL_IMAGE_WIDTH-1-x)*ORIGINAL_IMAGE_HEIGHT;   
   ThermalImage[offset]=value;
   if (value<min) min=value;
   if (value>max) max=value;
   // CRC
   CalculateCRC(crc,*((unsigned char*)(d_ptr)+1));
   CalculateCRC(crc,*((unsigned char*)(d_ptr)));
  }
  if (crc!=crc_package)//CRC  
  {
   enabled=false;
   break;
  }

  crc=0;
  // 
  id=*d_ptr;
  id&=0x0FFF;
  CalculateCRC(crc,(id>>8)&0xFF);
  CalculateCRC(crc,id&0xFF);
  d_ptr++;
  crc_package=*d_ptr;  
  CalculateCRC(crc,0);
  CalculateCRC(crc,0);
  d_ptr++;
  for(x=ORIGINAL_IMAGE_WIDTH/2;x<ORIGINAL_IMAGE_WIDTH;x++,d_ptr++)
  {   
   unsigned short value=*d_ptr;
   unsigned long offset=y+(ORIGINAL_IMAGE_WIDTH-1-x)*ORIGINAL_IMAGE_HEIGHT;   
   ThermalImage[offset]=value;
   if (value<min) min=value;
   if (value>max) max=value;
   // CRC
   CalculateCRC(crc,*((unsigned char*)(d_ptr)+1));
   CalculateCRC(crc,*((unsigned char*)(d_ptr)));
  }
  if (crc!=crc_package)//CRC  
  {
   enabled=false;
   break;
  }
 }
 if (enabled==true)
 {
  //  
  long delta=max-min;
  if (delta==0) delta=1;
  for(x=0;x<ORIGINAL_IMAGE_WIDTH;x++)
  {
   for(y=0;y<ORIGINAL_IMAGE_HEIGHT;y++)
   {
    long offset=x+y*ORIGINAL_IMAGE_WIDTH;
    long value=ThermalImage[offset];
    value-=min;
    value=(value*255)/delta;
    offset=x+y*ORIGINAL_IMAGE_WIDTH;
    value&=0xff;
	unsigned long color=0;
    color|=0xff;
	color<<=8;
    color|=ColorMap_R[value];
	color<<=8;
    color|=ColorMap_G[value];
	color<<=8;
    color|=ColorMap_B[value];
	ColorImage[offset]=color;
   }
  }
  FrameIndex++;    
 }
 return(true);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::Open(void)
{
 Close();
 AddLog("Open!\r\n");
 char completeDeviceName_iAP[MAX_DEVPATH_LENGTH];
 char completeDeviceName_Frame[MAX_DEVPATH_LENGTH];
 char completeDeviceName_FileIO[MAX_DEVPATH_LENGTH];
 if (GetDevicePath((LPGUID)&GUID_DEVINTERFACE_FLIRONEGEN2_IAP,completeDeviceName_iAP,sizeof(completeDeviceName_iAP))==false)//     
 {
  AddLog("[]    iAP  GUID!\r\n");
  return(false);
 }
 if (GetDevicePath((LPGUID)&GUID_DEVINTERFACE_FLIRONEGEN2_FRAME,completeDeviceName_Frame,sizeof(completeDeviceName_Frame))==false)//     
 {
  AddLog("[]    Frame  GUID!\r\n");
  return(false);
 }
 if (GetDevicePath((LPGUID)&GUID_DEVINTERFACE_FLIRONEGEN2_FILEIO,completeDeviceName_FileIO,sizeof(completeDeviceName_FileIO))==false)//     
 {
  AddLog("[]    Frame  GUID!\r\n");
  return(false);
 }
 AddLog("iAP: ");
 AddLog(completeDeviceName_iAP);
 AddLog("\r\n");
 AddLog("Frame: ");
 AddLog(completeDeviceName_Frame);
 AddLog("\r\n");
 AddLog("FileIO: ");
 AddLog(completeDeviceName_FileIO);
 AddLog("\r\n");
 //  
 hFile_iAP=CreateFile(completeDeviceName_iAP,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_WRITE|FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,NULL);
 hFile_Frame=CreateFile(completeDeviceName_Frame,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_WRITE|FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,NULL);
 hFile_FileIO=CreateFile(completeDeviceName_FileIO,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_WRITE|FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,NULL);

 if (hFile_iAP==INVALID_HANDLE_VALUE)
 {
  AddLog("[]    iAP!\r\n");
  Close();
  return(false);
 }
 if (hFile_Frame==INVALID_HANDLE_VALUE)
 {
 AddLog("[]    Frame!\r\n");
 Close();
 return(false);
 }
 if (hFile_FileIO==INVALID_HANDLE_VALUE)
 {
 AddLog("[]    FileIO!\r\n");
 Close();
 return(false);
 }
 //   

 if (SendControlCommand(IOCTL_FLIR_ONE_GEN2_FRAME_STOP)==false)
 {
  Close();
  return(false);  
 }
 if (SendControlCommand(IOCTL_FLIR_ONE_GEN2_FILEIO_STOP)==false)
 {
  Close();
  return(false);  
 }
 if (SendControlCommand(IOCTL_FLIR_ONE_GEN2_FILEIO_START)==false)
 {
  Close();
  return(false);  
 }
 if (SendControlCommand(IOCTL_FLIR_ONE_GEN2_FRAME_START)==false)
 {
  Close();
  return(false);  
 }

 return(true);
}
//----------------------------------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::Close(void)
{
 AddLog("Close!\r\n");
 if (hFile_FileIO!=INVALID_HANDLE_VALUE)
 {
  AddLog("FileIO .\r\n");
  CloseHandle(hFile_FileIO);
 }
 if (hFile_Frame!=INVALID_HANDLE_VALUE)
 {
  AddLog("Frame .\r\n");
  CloseHandle(hFile_Frame);
 }
 if (hFile_iAP!=INVALID_HANDLE_VALUE)
 {
  AddLog("iAP .\r\n");
  CloseHandle(hFile_iAP);
 }
 hFile_FileIO=INVALID_HANDLE_VALUE;
 hFile_Frame=INVALID_HANDLE_VALUE;
 hFile_iAP=INVALID_HANDLE_VALUE;
 Start_FileIO=true;
 Start_Frame=true;
 Start_iAP=true;
}
//----------------------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::Processing(void)
{
 if (hFile_FileIO==INVALID_HANDLE_VALUE) return(false);
 if (hFile_Frame==INVALID_HANDLE_VALUE) return(false);
 if (hFile_iAP==INVALID_HANDLE_VALUE) return(false);
 unsigned long readen; 
 //  
 if (Start_FileIO==true)// 
 {
  if (ReadFile(hFile_FileIO,Data,SizeOfData,&readen,&Overlapped_FileIO)==FALSE)//
  {   
   if (GetLastError()!=ERROR_IO_PENDING) 
   {
    AddLog("FileIO read error!\r\n");
	return(false);
   }
  }
  Start_FileIO=false;
 }
 if (Start_FileIO==false)//  
 {
  if (WaitForSingleObject(Overlapped_FileIO.hEvent,0)==WAIT_OBJECT_0)// 
  {
   AddLog("Event FileIO!\r\n");
   if (GetOverlappedResult(hFile_FileIO,&Overlapped_FileIO,&readen,TRUE)==TRUE)
   {
    Start_FileIO=true;//  
	AddLog("FileIO data!\r\n");
   }
  }
 }

 if (Start_iAP==true)// 
 {
  if (ReadFile(hFile_iAP,Data,SizeOfData,&readen,&Overlapped_iAP)==FALSE)//
  {
   if (GetLastError()!=ERROR_IO_PENDING) 
   {
    AddLog("iAP read error!\r\n");
    return(false);
   }
  }
  Start_iAP=false;
 }
 if (Start_iAP==false)//  
 {
  if (WaitForSingleObject(Overlapped_iAP.hEvent,0)==WAIT_OBJECT_0)// 
  {
   AddLog("Event iAP!\r\n");
   if (GetOverlappedResult(hFile_iAP,&Overlapped_iAP,&readen,TRUE)==TRUE)
   {
    Start_iAP=true;//  
	AddLog("iAP data!\r\n");
   }
  }
 }

 if (Start_Frame==true)// 
 {
  if (ReadFile(hFile_Frame,Data,SizeOfData,&readen,&Overlapped_Frame)==FALSE)//
  {
   if (GetLastError()!=ERROR_IO_PENDING)
   {
    AddLog("Frame read error!\r\n");
    return(false);
   }
  }
  Start_Frame=false;
 }
 if (Start_Frame==false)//  
 {
  if (WaitForSingleObject(Overlapped_Frame.hEvent,0)==WAIT_OBJECT_0)// 
  {
   AddLog("Event Frame!\r\n");
   if (GetOverlappedResult(hFile_Frame,&Overlapped_Frame,&readen,TRUE)==TRUE)
   {
    Start_Frame=true;//  
	CreateImage((unsigned char*)Data,readen);
	AddLog("Frame data!\r\n");
   }
  }
 }

 return(true);
}

//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::LoadColorMap(char *filename)
{ 
 FILE *file=fopen(filename,"rb");
 if (file==NULL) return(false);
 unsigned char colormap[768];
 if (fread(colormap,sizeof(unsigned char),768,file)<768)
 {
 fclose(file);
 return(false);
 }
 fclose(file);
 for(long n=0;n<256;n++)
 {
 ColorMap_R[n]=colormap[n*3+0];
 ColorMap_G[n]=colormap[n*3+1];
 ColorMap_B[n]=colormap[n*3+2];
 }
 return(true);
}

//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
unsigned short* CFlirOneControl::GetThermalImagePtr(unsigned long &frame_index)
{
 frame_index=FrameIndex;
 return(ThermalImage);	
}
//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
unsigned long* CFlirOneControl::GetColorImagePtr(unsigned long &frame_index)
{
 frame_index=FrameIndex;
 return(ColorImage);	
}
//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::CopyColorImage(unsigned long*image_ptr,unsigned long size,unsigned long &index)
{ 
 if (size<COLOR_IMAGE_SIZE_LONG) return(false);//  
 unsigned long *i_ptr=GetColorImagePtr(index);
 memcpy(image_ptr,i_ptr,COLOR_IMAGE_SIZE_LONG*sizeof(unsigned long));
 return(true); 
}
//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::CopyThermalImage(unsigned short *image_ptr,unsigned long size,unsigned long &index)
{ 
 if (size<THERMAL_IMAGE_SIZE_SHORT) return(false);//  
 unsigned short *i_ptr=GetThermalImagePtr(index);
 memcpy(image_ptr,i_ptr,THERMAL_IMAGE_SIZE_SHORT*sizeof(unsigned short));
 return(true); 
}



//---------------------------------------------------------------------

